#!/usr/bin/env python

# Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
#
# Contact: Ruslan Mstoi <ruslan.mstoi@nokia.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301  USA.

import sys
import shutil
import os
import re
import glob
import subprocess
import atexit

LAB_DIR = "___pgp_sign_lab"

def cleanup():
    os.chdir("../")
    shutil.rmtree("./" + LAB_DIR)

def exec_cmd(command):
    '''Executes a command. Exits if command fails.'''
    print ("Running %s" % repr(command))

    p = subprocess.Popen("exec " + command,
                         stdout = sys.stdout,
                         stderr = subprocess.STDOUT,
                         shell = True)
    p.wait()

    if p.returncode:
        sys.exit("Giving up, because failed to: %s" % command)

def find_deb_name_from_changelog():
    '''Returns the name of the existing debian package that matches the
    changlog package name and version'''    
    fo = open("./debian/changelog")

    p = re.compile('(^\S+) \((.*)\)')
    for line in fo:
        m = p.match(line)

        if m:
            break
    else:
        sys.exit("Could not find version from changelog")

    name = m.group(1)
    version = m.group(2)
    pat = "../%s_%s*.deb" % (name, version)
    files = glob.glob(pat)

    if not files:
        sys.exit("Files mathing pattern %s not found!" % pat)

    return files[0] # other matches ignored

def main():
    if os.path.exists(LAB_DIR):
        sys.exit("%s exists!" % LAB_DIR)

    # first argument if specified is used as the deb package
    if len(sys.argv) > 1:
        fn_deb = sys.argv[1]

        if not os.path.isfile(fn_deb):
            sys.exit("%s is not a file!")

    # otherwise deb name is found from changelog
    else:
        fn_deb = find_deb_name_from_changelog()

    fn_deb = os.path.abspath(fn_deb)
    gpg_key_home = os.path.join(os.getcwd(), "deb_pgp_sign/.local_gnupg")
    gpg_default_key = "4E08BD44"
    atexit.register(cleanup)

    os.mkdir(LAB_DIR)
    os.chdir(LAB_DIR)

    # extract package content
    exec_cmd("/usr/bin/ar vx %s" % fn_deb)

    if os.path.isfile("_gpgsig"):
        sys.exit("Package already signed!")

    # deb content files in the correct order
    files = ["debian-binary", "control.tar.gz", "data.tar.gz"]
    if os.path.isfile("_aegis"):
        files.append("_aegis")
    files_str = " ".join(files)

    # create combined content
    exec_cmd("cat %s > combined-content" % files_str)

    # sign
    exec_cmd("/usr/bin/gpg --homedir=%s --batch --default-key %s -abs "
             "-o _gpgsig combined-content" % (gpg_key_home, gpg_default_key))

    # backup original package
    shutil.copy(fn_deb, fn_deb + ".unsigned")

    # generate signed package
    exec_cmd("/usr/bin/ar vrc %s %s _gpgsig" % (fn_deb, files_str))

if __name__ == "__main__":
        main()
